function inference_segmentation(modelname, whichtest, ind_images, infer_params, suffix, DATASET_NAME, printout)
% INPUT:
% modelname    ... name of the model
% whichtest    ... either 'train' or 'test' (default) -- which image set we
%                 want to run inference on
% dataset_name ... name of the dataset (eg, 'mrsc' or 'voc10' (not
%    functional yet))
% results will be stored to fullfile(RESULTS_PATH, modelname, whichtest, 'seg') directory
% ind_images   ... if you want inference to be run on just a subset of
%                  images (set it to [] if you want to run on all images)

if nargin < 2, whichtest = 'test'; end;
if nargin < 3, ind_images = []; end;
if nargin < 7, printout = 0; end;
dataset_globals;  % get ALLPATHS
if nargin < 4 || ~numel(infer_params) % take default (see dataset_globals)
    infer_params = INFER_PARAMS;
end;
if isfield(infer_params, 'eps')
    eps = infer_params.eps;
else
    eps = 1;
end;
if isfield(infer_params, 'c')
    c = infer_params.c;
else
    c = 60;
end;
if isfield(infer_params, 's')
    s = infer_params.s;
else
    s = 60;
end;
if isfield(infer_params, 'r')
    r = infer_params.r;
else
    r = 1e-5;
end;

if nargin < 5, suffix = sprintf('eps-%0.3f', eps); end;

[classes, ~] = getclassinfo(ALLPATHS.DATASET_NAME);
if numel(IGNORE_CLASSES)
   classes = filterclasses(classes, IGNORE_CLASSES);   % classes
end;

train_dir = fullfile(ALLPATHS.LEARNING_PATH, modelname);
data = load(fullfile(train_dir, 'params.mat'));
params = data.params;
if isfield(params.learnparams, 'featparams')
    featparams = params.learnparams.featparams;
    featparams = defaultfeatparams(featparams, params.ind_classes);
else
    featparams = defaultfeatparams([], params.ind_classes);
end;
[~, ~, gamma, NumStates, BinaryStates, ignore_classes, ~, ind_classes] = parse_params(params);
dataset_globals;
data = load(fullfile(train_dir, 'theta.mat'));
theta = data.theta;


uai_dir = fullfile(RESULTS_PATH, modelname, suffix, 'uai');
results_dir = fullfile(RESULTS_PATH, modelname, suffix, 'seg');
log_dir = fullfile(results_dir, 'log');
if ~exist(uai_dir, 'dir'), mkdir(uai_dir); else unix(sprintf('rm -fR %s', uai_dir)); mkdir(uai_dir); end;
if ~exist(results_dir, 'dir'), mkdir(results_dir); end;
if ~exist(log_dir, 'dir'), mkdir(log_dir); end;

whichtest(1) = upper(whichtest(1));

im_file = fullfile(ALLPATHS.DATA_PATH, [whichtest '.txt']);
im_file = fopen(im_file,'r');
T = textscan(im_file,'%s');
imlist = T{1};
imlist = getimnames(imlist); % extract just the image name form the image list
fclose(im_file);

if numel(ind_images)
   imlist = imlist(ind_images);
end;
imlist = getList(imlist, results_dir);

if ~isempty(imlist)
    % compute the features
    fprintf('Computing features...\n');
    [features, ~, CRFDATA] = computeFeatures(imlist, NumStates, ignore_classes, ind_classes, gamma, [], ALLPATHS, 0, featparams);
    % write features to UAI files
    fprintf('done!\n');
    fprintf('Writing uai files...\n');
    libCRFmatlab(features, [], theta, 0, 0, [], [], [], [], [], [uai_dir  '/']);
end;
fprintf('done!\n');

% run inference
mpiexecfile = fullfile(CODE_MPIDIR_PATH,'mpiexec');
dcBPfile = fullfile(CODE_DCBP_PATH, 'dcBP');

% write inference parameters
fid = fopen(fullfile(log_dir, 'log.txt'), 'w+');
fprintf(fid, sprintf('eps = %0.2f\n', eps)); 
fprintf(fid, sprintf('c = %d\n', c));
fprintf(fid, sprintf('s = %d\n', s));
fclose(fid);

disp('Running inference')
for i = 1 : length(imlist)
    imname = imlist{i};
    fprintf('   image %s (%d/%d)\n', imname, i, length(imlist));
    outfile = fullfile(results_dir, [imname '.mat']);
    if exist(outfile, 'file')
        fprintf('      file already exists, skipping!\n');
        continue;
    end;
    
    uainame = sprintf('out_%d', i-1);
    uaifile = fullfile(uai_dir, [uainame '.uai']);
    belieffile = fullfile(uai_dir, [imname '-localBelief.txt']);
    command = sprintf('%s -n 2 %s -e %0.0f -s %d -c %d -r %d -f %s -o %s', mpiexecfile, dcBPfile, eps, s, c, r, uaifile, belieffile);
    [status result] = system(command);
    if status ~=0
       fprintf('     Error, system command failed!\n');
       continue;
    end;
    parse_result(classes, ind_classes, belieffile, NumStates, BinaryStates, outfile, CRFDATA.vars(i), printout);
    dlmwrite(fullfile(log_dir, [imname '.txt']), result, '');
end;

fprintf('finished!\n');


function [k1, k2, gamma, NumStates, BinaryStates, ignore_classes, boxdirname, ind_classes] = parse_params(params)

k1 = params.k1;
k2 = params.k2;
gamma = params.gamma;
NumStates = params.NumStates;
BinaryStates = params.BinaryStates;
ignore_classes = params.ignore_classes;
boxdirname = params.boxdirname;
ind_classes = params.ind_classes;

function imlist = getList(imlist, results_dir)

files = dir(fullfile(results_dir, '*.mat'));
files = arrayfun(@(x)strrep(x.name, '.mat', ''), files, 'UniformOutput', false);

imlist = setdiff(imlist, files);